<?php

namespace Flat101\Megamenu\Block\Adminhtml\Category;

class Tree extends \Magento\Framework\View\Element\Template
{
    /**
     * @var \Flat101\Megamenu\Helper\Data
     */
    protected $helper;

    /**
     * @var \Magento\Backend\Model\UrlInterface
     */
    protected $urlBuilder;

    /**
     * @var \Flat101\Megamenu\Model\MenuFactory
     */
    protected $modelMenuFactory;

    /**
     * @var \Magento\Framework\App\ResourceConnection
     */
    protected $resource;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Flat101\Megamenu\Model\ConfigFactory
     */
    protected $configFactory;

    /**
     * @var \Flat101\Megamenu\Model\MenuStoresFactory
     */
    protected $menuStoresFactory;

    /**
     * Tree constructor.
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Flat101\Megamenu\Helper\Data $helper
     * @param \Magento\Backend\Model\UrlInterface $urlBuilder
     * @param \Flat101\Megamenu\Model\MenuFactory $modelMenuFactory
     * @param \Flat101\Megamenu\Model\ConfigFactory $configFactory
     * @param \Magento\Framework\App\ResourceConnection $resource
     * @param \Flat101\Megamenu\Model\MenuStoresFactory $menuStoresFactory
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Flat101\Megamenu\Helper\Data $helper,
        \Magento\Backend\Model\UrlInterface $urlBuilder,
        \Flat101\Megamenu\Model\MenuFactory $modelMenuFactory,
        \Flat101\Megamenu\Model\ConfigFactory $configFactory,
        \Magento\Framework\App\ResourceConnection $resource,
        \Flat101\Megamenu\Model\MenuStoresFactory $menuStoresFactory
    ) {
        $this->helper = $helper;
        $this->configFactory = $configFactory;
        $this->urlBuilder = $urlBuilder;
        $this->modelMenuFactory = $modelMenuFactory;
        $this->resource = $resource;
        $this->scopeConfig = $context->getScopeConfig();
        $this->menuStoresFactory = $menuStoresFactory;
        parent::__construct($context);
    }

    /**
     * @return \Flat101\Megamenu\Helper\Data
     */
    public function getHelperData()
    {
        return $this->helper;
    }

    /**
     * @return mixed|string
     */
    public function menuTree()
    {
        $modelId = $this->getRequest()->getParam('id');
        $store = $this->menuStoresFactory->create();
        $storeModel = $store->load($modelId);

        if (!$storeModel) {
            return $this->getDefaultMenu();
        }

        $storeId = $storeModel->getStoreId();

        if ($storeId == null) {
            $storeId = 0;
        }

        $menu = $this->getHelperData()->getMegaMenuConfig($storeId);

        if ($menu == '' || $menu == 'false') {
            $menu = $this->getDefaultMenu();
        }

        return $menu;
    }

    /**
     * @return string
     */
    protected function getDefaultMenu()
    {
        return '[{ "text" : "Root Menu", "id" : "root"}]';
    }

    /**
     * Retrieve message text
     *
     * @return string
     */
    public function getText()
    {
        $cacheTypes = 'Page Cache';
        $message = __('One or more of the Cache Types are invalidated: %1. ', $cacheTypes) . ' ';
        $url = $this->urlBuilder->getUrl('adminhtml/cache');
        $message .= __("Please go to <a href='%1'>Cache Management</a> and refresh cache types.", $url);
        return $message;
    }

    /**
     * @param $type
     * @return string
     */
    public function getNodeUrl($type)
    {
        return $this->urlBuilder->getUrl('megamenu/category/create', ['type' => $type]);
    }

    /**
     * @param $type
     * @return string
     */
    public function getMenuItemUrl($type)
    {
        return $this->urlBuilder->getUrl('megamenu/item/edit', ['type' => $type]);
    }

    /**
     * @return int
     */
    public function getStoreId()
    {
        $modelId = $this->getRequest()->getParam('id');
        $store = $this->menuStoresFactory->create();
        $storeModel = $store->load($modelId);
        if (!$storeModel) {
            return '';
        }

        return $storeModel->getStoreId();
    }
}
