<?php
/**
 * Flat101 Commerce Co.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://Flat101commerce.com/Flat101-Commerce-License.txt
 *
 * @category   Flat101
 * @package    Flat101_MegaMenu
 * @author     Extension Team
 * @copyright  Copyright (c) 2016-2017 Flat101 Commerce Co. ( http://Flat101commerce.com )
 * @license    http://Flat101commerce.com/Flat101-Commerce-License.txtory
 */
namespace Flat101\Megamenu\Controller\Adminhtml\Category;

class MassDelete extends \Magento\Backend\App\Action
{
    /**
     * Mass action filter
     *
     * @var \Magento\Ui\Component\MassAction\Filter
     */
    protected $filter;

    /**
     * @var \Flat101\Megamenu\Model\ResourceModel\MenuStores\CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var \Flat101\Megamenu\Model\MenuStoresFactory
     */
    protected $menuStoresFactory;

    /**
     * MassDelete constructor.
     * @param \Magento\Ui\Component\MassAction\Filter $filter
     * @param \Flat101\Megamenu\Model\ResourceModel\MenuStores\CollectionFactory $collectionFactory
     * @param \Flat101\Megamenu\Model\MenuStoresFactory $menuStoresFactory
     * @param \Magento\Backend\App\Action\Context $context
     */
    public function __construct(
        \Magento\Ui\Component\MassAction\Filter $filter,
        \Flat101\Megamenu\Model\ResourceModel\MenuStores\CollectionFactory $collectionFactory,
        \Flat101\Megamenu\Model\MenuStoresFactory $menuStoresFactory,
        \Magento\Backend\App\Action\Context $context
    ) {
        $this->filter = $filter;
        $this->collectionFactory = $collectionFactory;
        $this->menuStoresFactory = $menuStoresFactory;
        parent::__construct($context);
    }

    /**
     * Execute delete comment
     *
     * @return \Magento\Backend\Model\View\Result\Redirect
     */
    public function execute()
    {
        try {
            $collection = $this->filter->getCollection($this->collectionFactory->create());
            $i = 0;
            foreach ($collection as $item) {
                $menuModel = $this->menuStoresFactory->create();
                $this->loadMenuModel($menuModel, $item);
                $this->deleteMenuModel($menuModel);
                $i++;
            }

            $this->messageManager->addSuccessMessage(
                __('A total of %1 record(s) have been deleted.', $i)
            );
        } catch (\Exception $e) {
            $this->messageManager->addErrorMessage($e->getMessage());
        }
        $resultRedirect = $this->resultFactory->create(\Magento\Framework\Controller\ResultFactory::TYPE_REDIRECT);
        return $resultRedirect->setPath('*/*/');
    }

    /**
     * @param $menuModel
     * @param $item
     * @return mixed
     */
    protected function loadMenuModel($menuModel, $item)
    {
        return $menuModel->load($item->getId());
    }

    /**
     * @param $menuModel
     * @return mixed
     */
    protected function deleteMenuModel($menuModel)
    {
        return $menuModel->delete();
    }
}
